package java.util;

import sun.util.calendar.CalendarSystem;
import java.io.IOException;
import java.io.ObjectInputStream;
import sun.util.calendar.CalendarUtils;
import sun.util.calendar.ZoneInfo;
import sun.util.calendar.BaseCalendar;
import sun.util.locale.provider.CalendarDataUtility;
import sun.util.calendar.CalendarDate;
import sun.util.calendar.AbstractCalendar;
import sun.util.calendar.Era;
import sun.util.calendar.Gregorian;
import sun.util.calendar.LocalGregorianCalendar;

class JapaneseImperialCalendar extends Calendar
{
    public static final int BEFORE_MEIJI = 0;
    public static final int MEIJI = 1;
    public static final int TAISHO = 2;
    public static final int SHOWA = 3;
    public static final int HEISEI = 4;
    private static final int REIWA = 5;
    private static final int EPOCH_OFFSET = 719163;
    private static final int EPOCH_YEAR = 1970;
    private static final int ONE_SECOND = 1000;
    private static final int ONE_MINUTE = 60000;
    private static final int ONE_HOUR = 3600000;
    private static final long ONE_DAY = 86400000L;
    private static final long ONE_WEEK = 604800000L;
    private static final LocalGregorianCalendar jcal;
    private static final Gregorian gcal;
    private static final Era BEFORE_MEIJI_ERA;
    private static final Era[] eras;
    private static final long[] sinceFixedDates;
    private static final int currentEra;
    static final int[] MIN_VALUES;
    static final int[] LEAST_MAX_VALUES;
    static final int[] MAX_VALUES;
    private static final long serialVersionUID = -3364572813905467929L;
    private transient LocalGregorianCalendar.Date jdate;
    private transient int[] zoneOffsets;
    private transient int[] originalFields;
    private transient long cachedFixedDate;
    
    JapaneseImperialCalendar(TimeZone zone, Locale locale) {
        super(zone, locale);
        this.cachedFixedDate = Long.MIN_VALUE;
        this.jdate = JapaneseImperialCalendar.jcal.newCalendarDate(zone);
        this.setTimeInMillis(System.currentTimeMillis());
    }
    
    JapaneseImperialCalendar(TimeZone zone, Locale locale, boolean b) {
        super(zone, locale);
        this.cachedFixedDate = Long.MIN_VALUE;
        this.jdate = JapaneseImperialCalendar.jcal.newCalendarDate(zone);
    }
    
    @Override
    public String getCalendarType() {
        return "japanese";
    }
    
    @Override
    public boolean equals(Object o) {
        return o instanceof JapaneseImperialCalendar && super.equals(o);
    }
    
    @Override
    public int hashCode() {
        return super.hashCode() ^ this.jdate.hashCode();
    }
    
    @Override
    public void add(int n, int n2) {
        if (n2 == 0) {
            return;
        }
        if (n < 0 || n >= 15) {
            throw new IllegalArgumentException();
        }
        this.complete();
        if (n == 1) {
            LocalGregorianCalendar.Date date2 = (LocalGregorianCalendar.Date)this.jdate.clone();
            date2.addYear(n2);
            this.pinDayOfMonth(date2);
            this.set(0, getEraIndex(date2));
            this.set(1, date2.getYear());
            this.set(2, date2.getMonth() - 1);
            this.set(5, date2.getDayOfMonth());
        }
        else if (n == 2) {
            LocalGregorianCalendar.Date date = (LocalGregorianCalendar.Date)this.jdate.clone();
            date.addMonth(n2);
            this.pinDayOfMonth(date);
            this.set(0, getEraIndex(date));
            this.set(1, date.getYear());
            this.set(2, date.getMonth() - 1);
            this.set(5, date.getDayOfMonth());
        }
        else if (n == 0) {
            int n9 = this.internalGet(0) + n2;
            if (n9 < 0) {
                n9 = 0;
            }
            else if (n9 > JapaneseImperialCalendar.eras.length - 1) {
                n9 = JapaneseImperialCalendar.eras.length - 1;
            }
            this.set(0, n9);
        }
        else {
            long n3 = (long)n2;
            long n4 = 0L;
            switch (n) {
                case 10:
                case 11: {
                    n3 *= 3600000L;
                    break;
                }
                case 12: {
                    n3 *= 60000L;
                    break;
                }
                case 13: {
                    n3 *= 1000L;
                    break;
                }
                case 14: {
                    break;
                }
                case 3:
                case 4:
                case 8: {
                    n3 *= 7L;
                    break;
                }
                case 5:
                case 6:
                case 7: {
                    break;
                }
                case 9: {
                    n3 = (long)(n2 / 2);
                    n4 = (long)(12 * (n2 % 2));
                    break;
                }
            }
            if (n >= 10) {
                this.setTimeInMillis(this.time + n3);
                return;
            }
            long cachedFixedDate = this.cachedFixedDate;
            long n5 = (((n4 + (long)this.internalGet(11)) * 60L + (long)this.internalGet(12)) * 60L + (long)this.internalGet(13)) * 1000L + (long)this.internalGet(14);
            if (n5 >= 86400000L) {
                ++cachedFixedDate;
                n5 -= 86400000L;
            }
            else if (n5 < 0L) {
                --cachedFixedDate;
                n5 += 86400000L;
            }
            long n6 = cachedFixedDate + n3;
            int n7 = this.internalGet(15) + this.internalGet(16);
            this.setTimeInMillis((n6 - 719163L) * 86400000L + n5 - (long)n7);
            int n8 = n7 - (this.internalGet(15) + this.internalGet(16));
            if (n8 != 0) {
                this.setTimeInMillis(this.time + (long)n8);
                if (this.cachedFixedDate != n6) {
                    this.setTimeInMillis(this.time - (long)n8);
                }
            }
        }
        return;
    }
    
    @Override
    public void roll(int n, boolean b) {
        this.roll(n, b ? 1 : -1);
    }
    
    @Override
    public void roll(int n, int n2) {
        if (n2 == 0) {
            return;
        }
        if (n < 0 || n >= 15) {
            throw new IllegalArgumentException();
        }
        this.complete();
        int n3 = this.getMinimum(n);
        int n4 = this.getMaximum(n);
        switch (n) {
            case 0:
            case 9:
            case 12:
            case 13:
            case 14: {
                break;
            }
            case 10:
            case 11: {
                int n5 = n4 + 1;
                int internalGet = this.internalGet(n);
                int n6 = (internalGet + n2) % n5;
                if (n6 < 0) {
                    n6 += n5;
                }
                this.time += (long)(3600000 * (n6 - internalGet));
                LocalGregorianCalendar.Date calendarDate = JapaneseImperialCalendar.jcal.getCalendarDate(this.time, this.getZone());
                if (this.internalGet(5) != calendarDate.getDayOfMonth()) {
                    calendarDate.setEra(this.jdate.getEra());
                    calendarDate.setDate(this.internalGet(1), this.internalGet(2) + 1, this.internalGet(5));
                    if (n == 10) {
                        assert this.internalGet(9) == 1;
                        calendarDate.addHours(12);
                    }
                    this.time = JapaneseImperialCalendar.jcal.getTime(calendarDate);
                }
                int hours = calendarDate.getHours();
                this.internalSet(n, hours % n5);
                if (n == 10) {
                    this.internalSet(11, hours);
                }
                else {
                    this.internalSet(9, hours / 12);
                    this.internalSet(10, hours % 12);
                }
                int zoneOffset = calendarDate.getZoneOffset();
                int daylightSaving = calendarDate.getDaylightSaving();
                this.internalSet(15, zoneOffset - daylightSaving);
                this.internalSet(16, daylightSaving);
                return;
            }
            case 1: {
                n3 = this.getActualMinimum(n);
                n4 = this.getActualMaximum(n);
                break;
            }
            case 2: {
                if (!this.isTransitionYear(this.jdate.getNormalizedYear())) {
                    int year = this.jdate.getYear();
                    if (year == this.getMaximum(1)) {
                        LocalGregorianCalendar.Date calendarDate5 = JapaneseImperialCalendar.jcal.getCalendarDate(this.time, this.getZone());
                        LocalGregorianCalendar.Date calendarDate6 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MAX_VALUE, this.getZone());
                        int n9 = calendarDate6.getMonth() - 1;
                        int rolledValue3 = getRolledValue(this.internalGet(n), n2, n3, n9);
                        if (rolledValue3 == n9) {
                            calendarDate5.addYear(-400);
                            calendarDate5.setMonth(rolledValue3 + 1);
                            if (calendarDate5.getDayOfMonth() > calendarDate6.getDayOfMonth()) {
                                calendarDate5.setDayOfMonth(calendarDate6.getDayOfMonth());
                                JapaneseImperialCalendar.jcal.normalize(calendarDate5);
                            }
                            if (calendarDate5.getDayOfMonth() == calendarDate6.getDayOfMonth() && calendarDate5.getTimeOfDay() > calendarDate6.getTimeOfDay()) {
                                calendarDate5.setMonth(rolledValue3 + 1);
                                calendarDate5.setDayOfMonth(calendarDate6.getDayOfMonth() - 1);
                                JapaneseImperialCalendar.jcal.normalize(calendarDate5);
                                rolledValue3 = calendarDate5.getMonth() - 1;
                            }
                            this.set(5, calendarDate5.getDayOfMonth());
                        }
                        this.set(2, rolledValue3);
                    }
                    else if (year == this.getMinimum(1)) {
                        LocalGregorianCalendar.Date calendarDate3 = JapaneseImperialCalendar.jcal.getCalendarDate(this.time, this.getZone());
                        LocalGregorianCalendar.Date calendarDate4 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MIN_VALUE, this.getZone());
                        int n8 = calendarDate4.getMonth() - 1;
                        int rolledValue2 = getRolledValue(this.internalGet(n), n2, n8, n4);
                        if (rolledValue2 == n8) {
                            calendarDate3.addYear(400);
                            calendarDate3.setMonth(rolledValue2 + 1);
                            if (calendarDate3.getDayOfMonth() < calendarDate4.getDayOfMonth()) {
                                calendarDate3.setDayOfMonth(calendarDate4.getDayOfMonth());
                                JapaneseImperialCalendar.jcal.normalize(calendarDate3);
                            }
                            if (calendarDate3.getDayOfMonth() == calendarDate4.getDayOfMonth() && calendarDate3.getTimeOfDay() < calendarDate4.getTimeOfDay()) {
                                calendarDate3.setMonth(rolledValue2 + 1);
                                calendarDate3.setDayOfMonth(calendarDate4.getDayOfMonth() + 1);
                                JapaneseImperialCalendar.jcal.normalize(calendarDate3);
                                rolledValue2 = calendarDate3.getMonth() - 1;
                            }
                            this.set(5, calendarDate3.getDayOfMonth());
                        }
                        this.set(2, rolledValue2);
                    }
                    else {
                        int n7 = (this.internalGet(2) + n2) % 12;
                        if (n7 < 0) {
                            n7 += 12;
                        }
                        this.set(2, n7);
                        int monthLength = this.monthLength(n7);
                        if (this.internalGet(5) > monthLength) {
                            this.set(5, monthLength);
                        }
                    }
                }
                else {
                    int eraIndex = getEraIndex(this.jdate);
                    CalendarDate calendarDate2 = null;
                    if (this.jdate.getYear() == 1) {
                        calendarDate2 = JapaneseImperialCalendar.eras[eraIndex].getSinceDate();
                        n3 = calendarDate2.getMonth() - 1;
                    }
                    else if (eraIndex < JapaneseImperialCalendar.eras.length - 1) {
                        calendarDate2 = JapaneseImperialCalendar.eras[eraIndex + 1].getSinceDate();
                        if (calendarDate2.getYear() == this.jdate.getNormalizedYear()) {
                            n4 = calendarDate2.getMonth() - 1;
                            if (calendarDate2.getDayOfMonth() == 1) {
                                --n4;
                            }
                        }
                    }
                    if (n3 == n4) {
                        return;
                    }
                    int rolledValue = getRolledValue(this.internalGet(n), n2, n3, n4);
                    this.set(2, rolledValue);
                    if (rolledValue == n3) {
                        if ((calendarDate2.getMonth() != 1 || calendarDate2.getDayOfMonth() != 1) && this.jdate.getDayOfMonth() < calendarDate2.getDayOfMonth()) {
                            this.set(5, calendarDate2.getDayOfMonth());
                        }
                    }
                    else if (rolledValue == n4 && calendarDate2.getMonth() - 1 == rolledValue) {
                        int dayOfMonth = calendarDate2.getDayOfMonth();
                        if (this.jdate.getDayOfMonth() >= dayOfMonth) {
                            this.set(5, dayOfMonth - 1);
                        }
                    }
                }
                return;
            }
            case 3: {
                int normalizedYear = this.jdate.getNormalizedYear();
                n4 = this.getActualMaximum(3);
                this.set(7, this.internalGet(7));
                int internalGet2 = this.internalGet(3);
                int n10 = internalGet2 + n2;
                if (!this.isTransitionYear(this.jdate.getNormalizedYear())) {
                    int year2 = this.jdate.getYear();
                    if (year2 == this.getMaximum(1)) {
                        n4 = this.getActualMaximum(3);
                    }
                    else if (year2 == this.getMinimum(1)) {
                        n3 = this.getActualMinimum(3);
                        n4 = this.getActualMaximum(3);
                        if (n10 > n3 && n10 < n4) {
                            this.set(3, n10);
                            return;
                        }
                    }
                    if (n10 > n3 && n10 < n4) {
                        this.set(3, n10);
                        return;
                    }
                    long cachedFixedDate2 = this.cachedFixedDate;
                    long fixedDate = cachedFixedDate2 - (long)(7 * (internalGet2 - n3));
                    if (year2 != this.getMinimum(1)) {
                        if (JapaneseImperialCalendar.gcal.getYearFromFixedDate(fixedDate) != normalizedYear) {
                            ++n3;
                        }
                    }
                    else if (fixedDate < JapaneseImperialCalendar.jcal.getFixedDate(JapaneseImperialCalendar.jcal.getCalendarDate(Long.MIN_VALUE, this.getZone()))) {
                        ++n3;
                    }
                    if (JapaneseImperialCalendar.gcal.getYearFromFixedDate(cachedFixedDate2 + (long)(7 * (n4 - this.internalGet(3)))) != normalizedYear) {
                        --n4;
                        break;
                    }
                    break;
                }
                else {
                    long cachedFixedDate = this.cachedFixedDate;
                    long n11 = cachedFixedDate - (long)(7 * (internalGet2 - n3));
                    LocalGregorianCalendar.Date calendarDate7 = getCalendarDate(n11);
                    if (calendarDate7.getEra() != this.jdate.getEra() || calendarDate7.getYear() != this.jdate.getYear()) {
                        ++n3;
                    }
                    JapaneseImperialCalendar.jcal.getCalendarDateFromFixedDate(calendarDate7, cachedFixedDate + (long)(7 * (n4 - internalGet2)));
                    if (calendarDate7.getEra() != this.jdate.getEra() || calendarDate7.getYear() != this.jdate.getYear()) {
                        --n4;
                    }
                    LocalGregorianCalendar.Date calendarDate8 = getCalendarDate(n11 + (long)((getRolledValue(internalGet2, n2, n3, n4) - 1) * 7));
                    this.set(2, calendarDate8.getMonth() - 1);
                    this.set(5, calendarDate8.getDayOfMonth());
                    return;
                }
            }
            case 4: {
                int transitionYear = this.isTransitionYear(this.jdate.getNormalizedYear());
                int n12 = this.internalGet(7) - this.getFirstDayOfWeek();
                if (n12 < 0) {
                    n12 += 7;
                }
                long cachedFixedDate3 = this.cachedFixedDate;
                long fixedDateMonth1;
                int n13;
                if (!(transitionYear == 0)) {
                    fixedDateMonth1 = this.getFixedDateMonth1(this.jdate, cachedFixedDate3);
                    n13 = this.actualMonthLength();
                }
                else {
                    fixedDateMonth1 = cachedFixedDate3 - (long)this.internalGet(5) + 1L;
                    n13 = JapaneseImperialCalendar.jcal.getMonthLength(this.jdate);
                }
                long dayOfWeekDateOnOrBefore = AbstractCalendar.getDayOfWeekDateOnOrBefore(fixedDateMonth1 + 6L, this.getFirstDayOfWeek());
                if ((int)(dayOfWeekDateOnOrBefore - fixedDateMonth1) >= this.getMinimalDaysInFirstWeek()) {
                    dayOfWeekDateOnOrBefore -= 7L;
                }
                long n14 = dayOfWeekDateOnOrBefore + (long)((getRolledValue(this.internalGet(n), n2, 1, this.getActualMaximum(n)) - 1) * 7) + (long)n12;
                if (n14 < fixedDateMonth1) {
                    n14 = fixedDateMonth1;
                }
                else if (n14 >= fixedDateMonth1 + (long)n13) {
                    n14 = fixedDateMonth1 + (long)n13 - 1L;
                }
                this.set(5, (int)(n14 - fixedDateMonth1) + 1);
                return;
            }
            case 5: {
                if (!this.isTransitionYear(this.jdate.getNormalizedYear())) {
                    n4 = JapaneseImperialCalendar.jcal.getMonthLength(this.jdate);
                    break;
                }
                long fixedDateMonth2 = this.getFixedDateMonth1(this.jdate, this.cachedFixedDate);
                LocalGregorianCalendar.Date calendarDate9 = getCalendarDate(fixedDateMonth2 + (long)getRolledValue((int)(this.cachedFixedDate - fixedDateMonth2), n2, 0, this.actualMonthLength() - 1));
                assert getEraIndex(calendarDate9) == this.internalGetEra() && calendarDate9.getYear() == this.internalGet(1) && calendarDate9.getMonth() - 1 == this.internalGet(2);
                this.set(5, calendarDate9.getDayOfMonth());
                return;
            }
            case 6: {
                n4 = this.getActualMaximum(n);
                if (!this.isTransitionYear(this.jdate.getNormalizedYear())) {
                    break;
                }
                LocalGregorianCalendar.Date calendarDate10 = getCalendarDate(this.cachedFixedDate - (long)this.internalGet(6) + (long)getRolledValue(this.internalGet(6), n2, n3, n4));
                assert getEraIndex(calendarDate10) == this.internalGetEra() && calendarDate10.getYear() == this.internalGet(1);
                this.set(2, calendarDate10.getMonth() - 1);
                this.set(5, calendarDate10.getDayOfMonth());
                return;
            }
            case 7: {
                int normalizedYear2 = this.jdate.getNormalizedYear();
                if (!this.isTransitionYear(normalizedYear2) && !this.isTransitionYear(normalizedYear2 - 1)) {
                    int internalGet3 = this.internalGet(3);
                    if (internalGet3 > 1 && internalGet3 < 52) {
                        this.set(3, this.internalGet(3));
                        n4 = 7;
                        break;
                    }
                }
                n2 %= 7;
                if (n2 == 0) {
                    return;
                }
                long cachedFixedDate4 = this.cachedFixedDate;
                long dayOfWeekDateOnOrBefore2 = AbstractCalendar.getDayOfWeekDateOnOrBefore(cachedFixedDate4, this.getFirstDayOfWeek());
                long n15 = cachedFixedDate4 + (long)n2;
                if (n15 < dayOfWeekDateOnOrBefore2) {
                    n15 += 7L;
                }
                else if (n15 >= dayOfWeekDateOnOrBefore2 + 7L) {
                    n15 -= 7L;
                }
                LocalGregorianCalendar.Date calendarDate11 = getCalendarDate(n15);
                this.set(0, getEraIndex(calendarDate11));
                this.set(calendarDate11.getYear(), calendarDate11.getMonth() - 1, calendarDate11.getDayOfMonth());
                return;
            }
            case 8: {
                n3 = 1;
                if (!this.isTransitionYear(this.jdate.getNormalizedYear())) {
                    int internalGet4 = this.internalGet(5);
                    int monthLength2 = JapaneseImperialCalendar.jcal.getMonthLength(this.jdate);
                    int n19 = monthLength2 % 7;
                    n4 = monthLength2 / 7;
                    if ((internalGet4 - 1) % 7 < n19) {
                        ++n4;
                    }
                    this.set(7, this.internalGet(7));
                    break;
                }
                else {
                    long cachedFixedDate5 = this.cachedFixedDate;
                    long fixedDateMonth3 = this.getFixedDateMonth1(this.jdate, cachedFixedDate5);
                    int actualMonthLength = this.actualMonthLength();
                    int n16 = actualMonthLength % 7;
                    int n17 = actualMonthLength / 7;
                    int n18 = (int)(cachedFixedDate5 - fixedDateMonth3) % 7;
                    if (n18 < n16) {
                        ++n17;
                    }
                    this.set(5, getCalendarDate(fixedDateMonth3 + (long)((getRolledValue(this.internalGet(n), n2, n3, n17) - 1) * 7) + (long)n18).getDayOfMonth());
                    return;
                }
            }
        }
        this.set(n, getRolledValue(this.internalGet(n), n2, n3, n4));
        return;
    }
    
    @Override
    public String getDisplayName(int field, int style, Locale locale) {
        if (!this.checkDisplayNameParams(field, style, 1, 4, locale, 647)) {
            return null;
        }
        int value = this.get(field);
        if (field == 1 && (this.getBaseStyle(style) != 2 || value != 1 || this.get(0) == 0)) {
            return null;
        }
        String retrieveFieldValueName = CalendarDataUtility.retrieveFieldValueName(this.getCalendarType(), field, value, style, locale);
        if ((retrieveFieldValueName == null || retrieveFieldValueName.isEmpty()) && field == 0 && value < JapaneseImperialCalendar.eras.length) {
            Era era = JapaneseImperialCalendar.eras[value];
            retrieveFieldValueName = ((style == 1) ? era.getAbbreviation() : era.getName());
            return retrieveFieldValueName;
        }
        return retrieveFieldValueName;
    }
    
    @Override
    public Map<String, Integer> getDisplayNames(int field, int style, Locale locale) {
        if (!this.checkDisplayNameParams(field, style, 0, 4, locale, 647)) {
            return null;
        }
        Map<String, Integer> retrieveFieldValueNames = CalendarDataUtility.retrieveFieldValueNames(this.getCalendarType(), field, style, locale);
        if (retrieveFieldValueNames != null && field == 0) {
            int i = retrieveFieldValueNames.size();
            if (style == 0) {
                HashSet<Object> set = new HashSet();
                Iterator<Object> iterator = retrieveFieldValueNames.keySet().iterator();
                while (iterator.hasNext()) {
                    set.add(retrieveFieldValueNames.get((Object)iterator.next()));
                }
                i = set.size();
            }
            if (i < JapaneseImperialCalendar.eras.length) {
                int baseStyle = this.getBaseStyle(style);
                for (; i < JapaneseImperialCalendar.eras.length; ++i) {
                    Era era = JapaneseImperialCalendar.eras[i];
                    if (baseStyle == 0 || baseStyle == 1 || baseStyle == 4) {
                        retrieveFieldValueNames.put(era.getAbbreviation(), Integer.valueOf(i));
                    }
                    if (baseStyle == 0 || baseStyle == 2) {
                        retrieveFieldValueNames.put(era.getName(), Integer.valueOf(i));
                    }
                }
            }
        }
        return retrieveFieldValueNames;
    }
    
    @Override
    public int getMinimum(int n) {
        return JapaneseImperialCalendar.MIN_VALUES[n];
    }
    
    @Override
    public int getMaximum(int n) {
        switch (n) {
            case 1: {
                return Math.max(JapaneseImperialCalendar.LEAST_MAX_VALUES[1], JapaneseImperialCalendar.jcal.getCalendarDate(Long.MAX_VALUE, this.getZone()).getYear());
            }
            default: {
                return JapaneseImperialCalendar.MAX_VALUES[n];
            }
        }
    }
    
    @Override
    public int getGreatestMinimum(int n) {
        return (n == 1) ? 1 : JapaneseImperialCalendar.MIN_VALUES[n];
    }
    
    @Override
    public int getLeastMaximum(int n) {
        switch (n) {
            case 1: {
                return Math.min(JapaneseImperialCalendar.LEAST_MAX_VALUES[1], this.getMaximum(1));
            }
            default: {
                return JapaneseImperialCalendar.LEAST_MAX_VALUES[n];
            }
        }
    }
    
    @Override
    public int getActualMinimum(int n) {
        if (!Calendar.isFieldSet(14, n)) {
            return this.getMinimum(n);
        }
        int minimum = 0;
        LocalGregorianCalendar.Date calendarDate = JapaneseImperialCalendar.jcal.getCalendarDate(this.getNormalizedCalendar().getTimeInMillis(), this.getZone());
        int eraIndex = getEraIndex(calendarDate);
        switch (n) {
            case 1: {
                if (eraIndex > 0) {
                    minimum = 1;
                    LocalGregorianCalendar.Date calendarDate3 = JapaneseImperialCalendar.jcal.getCalendarDate(JapaneseImperialCalendar.eras[eraIndex].getSince(this.getZone()), this.getZone());
                    calendarDate.setYear(calendarDate3.getYear());
                    JapaneseImperialCalendar.jcal.normalize(calendarDate);
                    assert calendarDate.isLeapYear() == calendarDate3.isLeapYear();
                    if (this.getYearOffsetInMillis(calendarDate) < this.getYearOffsetInMillis(calendarDate3)) {
                        ++minimum;
                    }
                    break;
                }
                else {
                    minimum = this.getMinimum(n);
                    LocalGregorianCalendar.Date calendarDate2 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MIN_VALUE, this.getZone());
                    int year = calendarDate2.getYear();
                    if (year > 400) {
                        year -= 400;
                    }
                    calendarDate.setYear(year);
                    JapaneseImperialCalendar.jcal.normalize(calendarDate);
                    if (this.getYearOffsetInMillis(calendarDate) < this.getYearOffsetInMillis(calendarDate2)) {
                        ++minimum;
                    }
                    break;
                }
            }
            case 2: {
                if (eraIndex > 1 && calendarDate.getYear() == 1) {
                    LocalGregorianCalendar.Date calendarDate4 = JapaneseImperialCalendar.jcal.getCalendarDate(JapaneseImperialCalendar.eras[eraIndex].getSince(this.getZone()), this.getZone());
                    minimum = calendarDate4.getMonth() - 1;
                    if (calendarDate.getDayOfMonth() < calendarDate4.getDayOfMonth()) {
                        ++minimum;
                    }
                    break;
                }
                else {
                    break;
                }
            }
            case 3: {
                minimum = 1;
                LocalGregorianCalendar.Date calendarDate5 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MIN_VALUE, this.getZone());
                calendarDate5.addYear(400);
                JapaneseImperialCalendar.jcal.normalize(calendarDate5);
                calendarDate.setEra(calendarDate5.getEra());
                calendarDate.setYear(calendarDate5.getYear());
                JapaneseImperialCalendar.jcal.normalize(calendarDate);
                long fixedDate = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate5);
                long fixedDate2 = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate);
                long n2 = fixedDate2 - (long)(7 * (this.getWeekNumber(fixedDate, fixedDate2) - 1));
                if (n2 < fixedDate || (n2 == fixedDate && calendarDate.getTimeOfDay() < calendarDate5.getTimeOfDay())) {
                    ++minimum;
                    break;
                }
                break;
            }
        }
        return minimum;
    }
    
    @Override
    public int getActualMaximum(int index) {
        if ((0x1FE81 & 1 << index) != 0x0) {
            return this.getMaximum(index);
        }
        JapaneseImperialCalendar normalizedCalendar = this.getNormalizedCalendar();
        LocalGregorianCalendar.Date jdate = normalizedCalendar.jdate;
        int year = jdate.getNormalizedYear();
        switch (index) {
            case 2: {
                year = 11;
                if (this.isTransitionYear(jdate.getNormalizedYear())) {
                    int eraIndex = getEraIndex(jdate);
                    if (jdate.getYear() != 1) {
                        ++eraIndex;
                        assert eraIndex < JapaneseImperialCalendar.eras.length;
                    }
                    long n = JapaneseImperialCalendar.sinceFixedDates[eraIndex];
                    if (normalizedCalendar.cachedFixedDate < n) {
                        LocalGregorianCalendar.Date date = (LocalGregorianCalendar.Date)jdate.clone();
                        JapaneseImperialCalendar.jcal.getCalendarDateFromFixedDate(date, n - 1L);
                        year = date.getMonth() - 1;
                    }
                    break;
                }
                else {
                    LocalGregorianCalendar.Date calendarDate = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MAX_VALUE, this.getZone());
                    if (jdate.getEra() == calendarDate.getEra() && jdate.getYear() == calendarDate.getYear()) {
                        year = calendarDate.getMonth() - 1;
                    }
                    break;
                }
            }
            case 5: {
                year = JapaneseImperialCalendar.jcal.getMonthLength(jdate);
                break;
            }
            case 6: {
                if (this.isTransitionYear(jdate.getNormalizedYear())) {
                    int eraIndex2 = getEraIndex(jdate);
                    if (jdate.getYear() != 1) {
                        ++eraIndex2;
                        assert eraIndex2 < JapaneseImperialCalendar.eras.length;
                    }
                    long n2 = JapaneseImperialCalendar.sinceFixedDates[eraIndex2];
                    long cachedFixedDate = normalizedCalendar.cachedFixedDate;
                    Gregorian.Date calendarDate4 = JapaneseImperialCalendar.gcal.newCalendarDate(TimeZone.NO_TIMEZONE);
                    calendarDate4.setDate(jdate.getNormalizedYear(), 1, 1);
                    if (cachedFixedDate < n2) {
                        year = (int)(n2 - JapaneseImperialCalendar.gcal.getFixedDate(calendarDate4));
                    }
                    else {
                        calendarDate4.addYear(1);
                        year = (int)(JapaneseImperialCalendar.gcal.getFixedDate(calendarDate4) - n2);
                    }
                    break;
                }
                else {
                    LocalGregorianCalendar.Date calendarDate2 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MAX_VALUE, this.getZone());
                    if (jdate.getEra() == calendarDate2.getEra() && jdate.getYear() == calendarDate2.getYear()) {
                        long fixedDate2 = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate2);
                        year = (int)(fixedDate2 - this.getFixedDateJan1(calendarDate2, fixedDate2)) + 1;
                    }
                    else if (jdate.getYear() == this.getMinimum(1)) {
                        LocalGregorianCalendar.Date calendarDate3 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MIN_VALUE, this.getZone());
                        long fixedDate = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate3);
                        calendarDate3.addYear(1);
                        calendarDate3.setMonth(1).setDayOfMonth(1);
                        JapaneseImperialCalendar.jcal.normalize(calendarDate3);
                        year = (int)(JapaneseImperialCalendar.jcal.getFixedDate(calendarDate3) - fixedDate);
                    }
                    else {
                        year = JapaneseImperialCalendar.jcal.getYearLength(jdate);
                    }
                    break;
                }
            }
            case 3: {
                if (!this.isTransitionYear(jdate.getNormalizedYear())) {
                    LocalGregorianCalendar.Date calendarDate5 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MAX_VALUE, this.getZone());
                    if (jdate.getEra() == calendarDate5.getEra() && jdate.getYear() == calendarDate5.getYear()) {
                        long fixedDate5 = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate5);
                        year = this.getWeekNumber(this.getFixedDateJan1(calendarDate5, fixedDate5), fixedDate5);
                        break;
                    }
                    if (jdate.getEra() == null && jdate.getYear() == this.getMinimum(1)) {
                        LocalGregorianCalendar.Date calendarDate7 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MIN_VALUE, this.getZone());
                        calendarDate7.addYear(400);
                        JapaneseImperialCalendar.jcal.normalize(calendarDate7);
                        calendarDate5.setEra(calendarDate7.getEra());
                        calendarDate5.setDate(calendarDate7.getYear() + 1, 1, 1);
                        JapaneseImperialCalendar.jcal.normalize(calendarDate5);
                        long fixedDate3 = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate7);
                        long fixedDate4 = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate5);
                        long dayOfWeekDateOnOrBefore = AbstractCalendar.getDayOfWeekDateOnOrBefore(fixedDate4 + 6L, this.getFirstDayOfWeek());
                        if ((int)(dayOfWeekDateOnOrBefore - fixedDate4) >= this.getMinimalDaysInFirstWeek()) {
                            dayOfWeekDateOnOrBefore -= 7L;
                        }
                        year = this.getWeekNumber(fixedDate3, dayOfWeekDateOnOrBefore);
                        break;
                    }
                    else {
                        Gregorian.Date calendarDate6 = JapaneseImperialCalendar.gcal.newCalendarDate(TimeZone.NO_TIMEZONE);
                        calendarDate6.setDate(jdate.getNormalizedYear(), 1, 1);
                        int n3 = JapaneseImperialCalendar.gcal.getDayOfWeek(calendarDate6) - this.getFirstDayOfWeek();
                        if (n3 < 0) {
                            n3 += 7;
                        }
                        year = 52;
                        int n4 = n3 + this.getMinimalDaysInFirstWeek() - 1;
                        if (n4 == 6 || (jdate.isLeapYear() && (n4 == 5 || n4 == 12))) {
                            ++year;
                        }
                        break;
                    }
                }
                else {
                    if (normalizedCalendar == this) {
                        normalizedCalendar = (JapaneseImperialCalendar)normalizedCalendar.clone();
                    }
                    int actualMaximum = this.getActualMaximum(6);
                    normalizedCalendar.set(6, actualMaximum);
                    year = normalizedCalendar.get(3);
                    if (year == 1 && actualMaximum > 7) {
                        normalizedCalendar.add(3, -1);
                        year = normalizedCalendar.get(3);
                    }
                    break;
                }
            }
            case 4: {
                LocalGregorianCalendar.Date calendarDate8 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MAX_VALUE, this.getZone());
                if (jdate.getEra() != calendarDate8.getEra() || jdate.getYear() != calendarDate8.getYear()) {
                    Gregorian.Date calendarDate9 = JapaneseImperialCalendar.gcal.newCalendarDate(TimeZone.NO_TIMEZONE);
                    calendarDate9.setDate(jdate.getNormalizedYear(), jdate.getMonth(), 1);
                    int dayOfWeek = JapaneseImperialCalendar.gcal.getDayOfWeek(calendarDate9);
                    int monthLength = JapaneseImperialCalendar.gcal.getMonthLength(calendarDate9);
                    int n5 = dayOfWeek - this.getFirstDayOfWeek();
                    if (n5 < 0) {
                        n5 += 7;
                    }
                    int n6 = 7 - n5;
                    year = 3;
                    if (n6 >= this.getMinimalDaysInFirstWeek()) {
                        ++year;
                    }
                    int n7 = monthLength - (n6 + 21);
                    if (n7 > 0) {
                        ++year;
                        if (n7 > 7) {
                            ++year;
                        }
                    }
                }
                else {
                    long fixedDate6 = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate8);
                    year = this.getWeekNumber(fixedDate6 - (long)calendarDate8.getDayOfMonth() + 1L, fixedDate6);
                }
                break;
            }
            case 8: {
                int dayOfWeek2 = jdate.getDayOfWeek();
                BaseCalendar.Date date2 = (BaseCalendar.Date)jdate.clone();
                int monthLength2 = JapaneseImperialCalendar.jcal.getMonthLength(date2);
                date2.setDayOfMonth(1);
                JapaneseImperialCalendar.jcal.normalize(date2);
                int n8 = dayOfWeek2 - date2.getDayOfWeek();
                if (n8 < 0) {
                    n8 += 7;
                }
                year = (monthLength2 - n8 + 6) / 7;
                break;
            }
            case 1: {
                LocalGregorianCalendar.Date calendarDate10 = JapaneseImperialCalendar.jcal.getCalendarDate(normalizedCalendar.getTimeInMillis(), this.getZone());
                int eraIndex3 = getEraIndex(jdate);
                LocalGregorianCalendar.Date date3;
                if (eraIndex3 == JapaneseImperialCalendar.eras.length - 1) {
                    date3 = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MAX_VALUE, this.getZone());
                    year = date3.getYear();
                    if (year > 400) {
                        calendarDate10.setYear(year - 400);
                    }
                }
                else {
                    date3 = JapaneseImperialCalendar.jcal.getCalendarDate(JapaneseImperialCalendar.eras[eraIndex3 + 1].getSince(this.getZone()) - 1L, this.getZone());
                    year = date3.getYear();
                    calendarDate10.setYear(year);
                }
                JapaneseImperialCalendar.jcal.normalize(calendarDate10);
                if (this.getYearOffsetInMillis(calendarDate10) > this.getYearOffsetInMillis(date3)) {
                    --year;
                }
                break;
            }
            default: {
                throw new ArrayIndexOutOfBoundsException(index);
            }
        }
        return year;
    }
    
    private long getYearOffsetInMillis(CalendarDate date) {
        return (JapaneseImperialCalendar.jcal.getDayOfYear(date) - 1L) * 86400000L + date.getTimeOfDay() - (long)date.getZoneOffset();
    }
    
    @Override
    public Object clone() {
        JapaneseImperialCalendar japaneseImperialCalendar = (JapaneseImperialCalendar)super.clone();
        japaneseImperialCalendar.jdate = (LocalGregorianCalendar.Date)this.jdate.clone();
        japaneseImperialCalendar.originalFields = null;
        japaneseImperialCalendar.zoneOffsets = null;
        return japaneseImperialCalendar;
    }
    
    @Override
    public TimeZone getTimeZone() {
        TimeZone timeZone = super.getTimeZone();
        this.jdate.setZone(timeZone);
        return timeZone;
    }
    
    @Override
    public void setTimeZone(TimeZone timeZone) {
        super.setTimeZone(timeZone);
        this.jdate.setZone(timeZone);
    }
    
    @Override
    protected void computeFields() {
        int setStateFields;
        if (this.isPartiallyNormalized()) {
            setStateFields = this.getSetStateFields();
            int n = ~setStateFields & 0x1FFFF;
            if (n != 0 || this.cachedFixedDate == Long.MIN_VALUE) {
                setStateFields |= this.computeFields(n, setStateFields & 0x18000);
                assert setStateFields == 131071;
            }
        }
        else {
            setStateFields = 131071;
            this.computeFields(setStateFields, 0);
        }
        this.setFieldsComputed(setStateFields);
    }
    
    private int computeFields(int n, int n2) {
        int n3 = 0;
        TimeZone zone = this.getZone();
        if (this.zoneOffsets == null) {
            this.zoneOffsets = new int[2];
        }
        if (n2 != 98304) {
            if ((zone instanceof ZoneInfo)) {
                n3 = ((ZoneInfo)zone).getOffsets(this.time, this.zoneOffsets);
            }
            else {
                n3 = zone.getOffset(this.time);
                this.zoneOffsets[0] = zone.getRawOffset();
                this.zoneOffsets[1] = n3 - this.zoneOffsets[0];
            }
        }
        if (n2 != 0) {
            if (Calendar.isFieldSet(n2, 15)) {
                this.zoneOffsets[0] = this.internalGet(15);
            }
            if (Calendar.isFieldSet(n2, 16)) {
                this.zoneOffsets[1] = this.internalGet(16);
            }
            n3 = this.zoneOffsets[0] + this.zoneOffsets[1];
        }
        long n4 = (long)n3 / 86400000L;
        int n5 = n3 % 86400000;
        long n6 = n4 + this.time / 86400000L;
        int i = n5 + (int)(this.time % 86400000L);
        if ((long)i >= 86400000L) {
            i = (int)((long)i - 86400000L);
            ++n6;
        }
        else {
            while (i < 0) {
                i = (int)((long)i + 86400000L);
                --n6;
            }
        }
        long n7 = n6 + 719163L;
        if (n7 != this.cachedFixedDate || n7 < 0L) {
            JapaneseImperialCalendar.jcal.getCalendarDateFromFixedDate(this.jdate, n7);
            this.cachedFixedDate = n7;
        }
        int eraIndex = getEraIndex(this.jdate);
        int year = this.jdate.getYear();
        this.internalSet(0, eraIndex);
        this.internalSet(1, year);
        int n8 = n | 0x3;
        int n9 = this.jdate.getMonth() - 1;
        int dayOfMonth = this.jdate.getDayOfMonth();
        if ((n & 0xA4) != 0x0) {
            this.internalSet(2, n9);
            this.internalSet(5, dayOfMonth);
            this.internalSet(7, this.jdate.getDayOfWeek());
            n8 |= 0xA4;
        }
        if ((n & 0x7E00) != 0x0) {
            if (i != 0) {
                int n10 = i / 3600000;
                this.internalSet(11, n10);
                this.internalSet(9, n10 / 12);
                this.internalSet(10, n10 % 12);
                int n11 = i % 3600000;
                this.internalSet(12, n11 / 60000);
                int n12 = n11 % 60000;
                this.internalSet(13, n12 / 1000);
                this.internalSet(14, n12 % 1000);
            }
            else {
                this.internalSet(11, 0);
                this.internalSet(9, 0);
                this.internalSet(10, 0);
                this.internalSet(12, 0);
                this.internalSet(13, 0);
                this.internalSet(14, 0);
            }
            n8 |= 0x7E00;
        }
        if ((n & 0x18000) != 0x0) {
            this.internalSet(15, this.zoneOffsets[0]);
            this.internalSet(16, this.zoneOffsets[1]);
            n8 |= 0x18000;
        }
        if ((n & 0x158) != 0x0) {
            int normalizedYear = this.jdate.getNormalizedYear();
            int transitionYear = this.isTransitionYear(this.jdate.getNormalizedYear());
            int n13;
            long n14;
            if (!(transitionYear == 0)) {
                n14 = this.getFixedDateJan1(this.jdate, n7);
                n13 = (int)(n7 - n14) + 1;
            }
            else if (normalizedYear == JapaneseImperialCalendar.MIN_VALUES[1]) {
                n14 = JapaneseImperialCalendar.jcal.getFixedDate(JapaneseImperialCalendar.jcal.getCalendarDate(Long.MIN_VALUE, this.getZone()));
                n13 = (int)(n7 - n14) + 1;
            }
            else {
                n13 = (int)JapaneseImperialCalendar.jcal.getDayOfYear(this.jdate);
                n14 = n7 - (long)n13 + 1L;
            }
            long n15 = transitionYear ? this.getFixedDateMonth1(this.jdate, n7) : (n7 - (long)dayOfMonth + 1L);
            this.internalSet(6, n13);
            this.internalSet(8, (dayOfMonth - 1) / 7 + 1);
            int n16 = this.getWeekNumber(n14, n7);
            if (n16 == 0) {
                long n20 = n14 - 1L;
                LocalGregorianCalendar.Date calendarDate = getCalendarDate(n20);
                long n21;
                if (transitionYear == 0 && !this.isTransitionYear(calendarDate.getNormalizedYear())) {
                    n21 = n14 - 365L;
                    if (calendarDate.isLeapYear()) {
                        --n21;
                    }
                }
                else if (!(transitionYear == 0)) {
                    if (this.jdate.getYear() == 1) {
                        if (eraIndex > 5) {
                            CalendarDate sinceDate3 = JapaneseImperialCalendar.eras[eraIndex - 1].getSinceDate();
                            if (normalizedYear == sinceDate3.getYear()) {
                                calendarDate.setMonth(sinceDate3.getMonth()).setDayOfMonth(sinceDate3.getDayOfMonth());
                            }
                        }
                        else {
                            calendarDate.setMonth(1).setDayOfMonth(1);
                        }
                        JapaneseImperialCalendar.jcal.normalize(calendarDate);
                        n21 = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate);
                    }
                    else {
                        n21 = n14 - 365L;
                        if (calendarDate.isLeapYear()) {
                            --n21;
                        }
                    }
                }
                else {
                    CalendarDate sinceDate2 = JapaneseImperialCalendar.eras[getEraIndex(this.jdate)].getSinceDate();
                    calendarDate.setMonth(sinceDate2.getMonth()).setDayOfMonth(sinceDate2.getDayOfMonth());
                    JapaneseImperialCalendar.jcal.normalize(calendarDate);
                    n21 = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate);
                }
                n16 = this.getWeekNumber(n21, n20);
            }
            else if (transitionYear == 0) {
                if (n16 >= 52) {
                    long n19 = n14 + 365L;
                    if (this.jdate.isLeapYear()) {
                        ++n19;
                    }
                    long dayOfWeekDateOnOrBefore2 = AbstractCalendar.getDayOfWeekDateOnOrBefore(n19 + 6L, this.getFirstDayOfWeek());
                    if ((int)(dayOfWeekDateOnOrBefore2 - n19) >= this.getMinimalDaysInFirstWeek() && n7 >= dayOfWeekDateOnOrBefore2 - 7L) {
                        n16 = 1;
                    }
                }
            }
            else {
                LocalGregorianCalendar.Date date = (LocalGregorianCalendar.Date)this.jdate.clone();
                long n18;
                if (this.jdate.getYear() == 1) {
                    date.addYear(1);
                    date.setMonth(1).setDayOfMonth(1);
                    n18 = JapaneseImperialCalendar.jcal.getFixedDate(date);
                }
                else {
                    int n17 = getEraIndex(date) + 1;
                    CalendarDate sinceDate = JapaneseImperialCalendar.eras[n17].getSinceDate();
                    date.setEra(JapaneseImperialCalendar.eras[n17]);
                    date.setDate(1, sinceDate.getMonth(), sinceDate.getDayOfMonth());
                    JapaneseImperialCalendar.jcal.normalize(date);
                    n18 = JapaneseImperialCalendar.jcal.getFixedDate(date);
                }
                long dayOfWeekDateOnOrBefore = AbstractCalendar.getDayOfWeekDateOnOrBefore(n18 + 6L, this.getFirstDayOfWeek());
                if ((int)(dayOfWeekDateOnOrBefore - n18) >= this.getMinimalDaysInFirstWeek() && n7 >= dayOfWeekDateOnOrBefore - 7L) {
                    n16 = 1;
                }
            }
            this.internalSet(3, n16);
            this.internalSet(4, this.getWeekNumber(n15, n7));
            n8 |= 0x158;
        }
        return n8;
    }
    
    private int getWeekNumber(long n, long n2) {
        long dayOfWeekDateOnOrBefore = AbstractCalendar.getDayOfWeekDateOnOrBefore(n + 6L, this.getFirstDayOfWeek());
        int n3 = (int)(dayOfWeekDateOnOrBefore - n);
        assert n3 <= 7;
        if (n3 >= this.getMinimalDaysInFirstWeek()) {
            dayOfWeekDateOnOrBefore -= 7L;
        }
        int n4 = (int)(n2 - dayOfWeekDateOnOrBefore);
        if (n4 >= 0) {
            return n4 / 7 + 1;
        }
        return CalendarUtils.floorDivide(n4, 7) + 1;
    }
    
    @Override
    protected void computeTime() {
        if (!this.isLenient()) {
            if (this.originalFields == null) {
                this.originalFields = new int[17];
            }
            int i = 0;
            while (i < 17) {
                int internalGet = this.internalGet(i);
                if (this.isExternallySet(i) && (internalGet < this.getMinimum(i) || internalGet > this.getMaximum(i))) {
                    throw new IllegalArgumentException(Calendar.getFieldName(i));
                }
                this.originalFields[i] = internalGet;
                ++i;
            }
        }
        int selectFields = this.selectFields();
        int n;
        int internalGet2;
        if (this.isSet(0)) {
            n = this.internalGet(0);
            internalGet2 = (this.isSet(1) ? this.internalGet(1) : 1);
        }
        else if (this.isSet(1)) {
            n = JapaneseImperialCalendar.currentEra;
            internalGet2 = this.internalGet(1);
        }
        else {
            n = 3;
            internalGet2 = 45;
        }
        long n2 = 0L;
        long n3;
        if (Calendar.isFieldSet(selectFields, 11)) {
            n3 = n2 + (long)this.internalGet(11);
        }
        else {
            n3 = n2 + (long)this.internalGet(10);
            if (Calendar.isFieldSet(selectFields, 9)) {
                n3 += (long)(12 * this.internalGet(9));
            }
        }
        long n4 = ((n3 * 60L + (long)this.internalGet(12)) * 60L + (long)this.internalGet(13)) * 1000L + (long)this.internalGet(14);
        long n5;
        long n6;
        for (n5 = n4 / 86400000L, n6 = n4 % 86400000L; n6 < 0L; n6 += 86400000L, --n5) {}
        long wall = (n5 + this.getFixedDate(n, internalGet2, selectFields) - 719163L) * 86400000L + n6;
        TimeZone zone = this.getZone();
        if (this.zoneOffsets == null) {
            this.zoneOffsets = new int[2];
        }
        int n7 = selectFields & 0x18000;
        if (n7 != 98304) {
            if ((zone instanceof ZoneInfo)) {
                ((ZoneInfo)zone).getOffsetsByWall(wall, this.zoneOffsets);
            }
            else {
                zone.getOffsets(wall - (long)zone.getRawOffset(), this.zoneOffsets);
            }
        }
        if (n7 != 0) {
            if (Calendar.isFieldSet(n7, 15)) {
                this.zoneOffsets[0] = this.internalGet(15);
            }
            if (Calendar.isFieldSet(n7, 16)) {
                this.zoneOffsets[1] = this.internalGet(16);
            }
        }
        this.time = wall - (long)(this.zoneOffsets[0] + this.zoneOffsets[1]);
        int computeFields = this.computeFields(selectFields | this.getSetStateFields(), n7);
        if (!this.isLenient()) {
            for (int j = 0; j < 17; ++j) {
                if (this.isExternallySet(j) && this.originalFields[j] != this.internalGet(j)) {
                    int internalGet3 = this.internalGet(j);
                    System.arraycopy(this.originalFields, 0, this.fields, 0, this.fields.length);
                    throw new IllegalArgumentException(Calendar.getFieldName(j) + "=" + internalGet3 + ", expected " + this.originalFields[j]);
                }
            }
        }
        this.setFieldsNormalized(computeFields);
    }
    
    private long getFixedDate(int n, int year, int n2) {
        int internalGet = 0;
        int dayOfMonth = 1;
        if (Calendar.isFieldSet(n2, 2)) {
            internalGet = this.internalGet(2);
            if (internalGet > 11) {
                year += internalGet / 12;
                internalGet %= 12;
            }
            else if (internalGet < 0) {
                int[] r = { 0 };
                year += CalendarUtils.floorDivide(internalGet, 12, r);
                internalGet = r[0];
            }
        }
        else if (year == 1 && n != 0) {
            CalendarDate sinceDate = JapaneseImperialCalendar.eras[n].getSinceDate();
            internalGet = sinceDate.getMonth() - 1;
            dayOfMonth = sinceDate.getDayOfMonth();
        }
        if (year == JapaneseImperialCalendar.MIN_VALUES[1]) {
            LocalGregorianCalendar.Date calendarDate = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MIN_VALUE, this.getZone());
            int n3 = calendarDate.getMonth() - 1;
            if (internalGet < n3) {
                internalGet = n3;
            }
            if (internalGet == n3) {
                dayOfMonth = calendarDate.getDayOfMonth();
            }
        }
        LocalGregorianCalendar.Date calendarDate2 = JapaneseImperialCalendar.jcal.newCalendarDate(TimeZone.NO_TIMEZONE);
        calendarDate2.setEra((n > 0) ? JapaneseImperialCalendar.eras[n] : null);
        calendarDate2.setDate(year, internalGet + 1, dayOfMonth);
        JapaneseImperialCalendar.jcal.normalize(calendarDate2);
        long n4 = JapaneseImperialCalendar.jcal.getFixedDate(calendarDate2);
        if (Calendar.isFieldSet(n2, 2)) {
            if (Calendar.isFieldSet(n2, 5)) {
                if (this.isSet(5)) {
                    n4 = n4 + (long)this.internalGet(5) - (long)dayOfMonth;
                }
            }
            else if (Calendar.isFieldSet(n2, 4)) {
                long n7 = AbstractCalendar.getDayOfWeekDateOnOrBefore(n4 + 6L, this.getFirstDayOfWeek());
                if (n7 - n4 >= (long)this.getMinimalDaysInFirstWeek()) {
                    n7 -= 7L;
                }
                if (Calendar.isFieldSet(n2, 7)) {
                    n7 = AbstractCalendar.getDayOfWeekDateOnOrBefore(n7 + 6L, this.internalGet(7));
                }
                n4 = n7 + (long)(7 * (this.internalGet(4) - 1));
            }
            else {
                int n6;
                if (Calendar.isFieldSet(n2, 7)) {
                    n6 = this.internalGet(7);
                }
                else {
                    n6 = this.getFirstDayOfWeek();
                }
                int internalGet3;
                if (Calendar.isFieldSet(n2, 8)) {
                    internalGet3 = this.internalGet(8);
                }
                else {
                    internalGet3 = 1;
                }
                if (internalGet3 >= 0) {
                    n4 = AbstractCalendar.getDayOfWeekDateOnOrBefore(n4 + (long)(7 * internalGet3) - 1L, n6);
                }
                else {
                    n4 = AbstractCalendar.getDayOfWeekDateOnOrBefore(n4 + (long)(this.monthLength(internalGet, year) + 7 * (internalGet3 + 1)) - 1L, n6);
                }
            }
        }
        else if (Calendar.isFieldSet(n2, 6)) {
            if (this.isTransitionYear(calendarDate2.getNormalizedYear())) {
                n4 = this.getFixedDateJan1(calendarDate2, n4);
            }
            n4 = n4 + (long)this.internalGet(6) - 1L;
        }
        else {
            long n5 = AbstractCalendar.getDayOfWeekDateOnOrBefore(n4 + 6L, this.getFirstDayOfWeek());
            if (n5 - n4 >= (long)this.getMinimalDaysInFirstWeek()) {
                n5 -= 7L;
            }
            if (Calendar.isFieldSet(n2, 7)) {
                int internalGet2 = this.internalGet(7);
                if (internalGet2 != this.getFirstDayOfWeek()) {
                    n5 = AbstractCalendar.getDayOfWeekDateOnOrBefore(n5 + 6L, internalGet2);
                }
            }
            n4 = n5 + 7L * ((long)this.internalGet(3) - 1L);
        }
        return n4;
    }
    
    private long getFixedDateJan1(LocalGregorianCalendar.Date date, long n) {
        date.getEra();
        if (date.getEra() != null && date.getYear() == 1) {
            int i = getEraIndex(date);
            while (i > 0) {
                long fixedDate = JapaneseImperialCalendar.gcal.getFixedDate(JapaneseImperialCalendar.eras[i].getSinceDate());
                if (fixedDate > n) {
                    --i;
                }
                else {
                    return fixedDate;
                }
            }
        }
        Gregorian.Date calendarDate = JapaneseImperialCalendar.gcal.newCalendarDate(TimeZone.NO_TIMEZONE);
        calendarDate.setDate(date.getNormalizedYear(), 1, 1);
        return JapaneseImperialCalendar.gcal.getFixedDate(calendarDate);
    }
    
    private long getFixedDateMonth1(LocalGregorianCalendar.Date date, long n) {
        int transitionEraIndex = getTransitionEraIndex(date);
        if (transitionEraIndex != -1) {
            long n2 = JapaneseImperialCalendar.sinceFixedDates[transitionEraIndex];
            if (n2 <= n) {
                return n2;
            }
        }
        return n - (long)date.getDayOfMonth() + 1L;
    }
    
    private static LocalGregorianCalendar.Date getCalendarDate(long fixedDate) {
        LocalGregorianCalendar.Date calendarDate = JapaneseImperialCalendar.jcal.newCalendarDate(TimeZone.NO_TIMEZONE);
        JapaneseImperialCalendar.jcal.getCalendarDateFromFixedDate(calendarDate, fixedDate);
        return calendarDate;
    }
    
    private int monthLength(int n, int gregorianYear) {
        return CalendarUtils.isGregorianLeapYear(gregorianYear) ? GregorianCalendar.LEAP_MONTH_LENGTH[n] : GregorianCalendar.MONTH_LENGTH[n];
    }
    
    private int monthLength(int n) {
        assert !(!this.jdate.isNormalized());
        return this.jdate.isLeapYear() ? GregorianCalendar.LEAP_MONTH_LENGTH[n] : GregorianCalendar.MONTH_LENGTH[n];
    }
    
    private int actualMonthLength() {
        int monthLength = JapaneseImperialCalendar.jcal.getMonthLength(this.jdate);
        int transitionEraIndex = getTransitionEraIndex(this.jdate);
        if (transitionEraIndex == -1) {
            long n = JapaneseImperialCalendar.sinceFixedDates[transitionEraIndex];
            CalendarDate sinceDate = JapaneseImperialCalendar.eras[transitionEraIndex].getSinceDate();
            if (n <= this.cachedFixedDate) {
                monthLength -= sinceDate.getDayOfMonth() - 1;
            }
            else {
                monthLength = sinceDate.getDayOfMonth() - 1;
            }
        }
        return monthLength;
    }
    
    private static int getTransitionEraIndex(LocalGregorianCalendar.Date date) {
        int eraIndex = getEraIndex(date);
        CalendarDate sinceDate = JapaneseImperialCalendar.eras[eraIndex].getSinceDate();
        if (sinceDate.getYear() == date.getNormalizedYear() && sinceDate.getMonth() == date.getMonth()) {
            return eraIndex;
        }
        if (eraIndex < JapaneseImperialCalendar.eras.length - 1) {
            CalendarDate sinceDate2 = JapaneseImperialCalendar.eras[++eraIndex].getSinceDate();
            if (sinceDate2.getYear() == date.getNormalizedYear() && sinceDate2.getMonth() == date.getMonth()) {
                return eraIndex;
            }
        }
        return -1;
    }
    
    private boolean isTransitionYear(int n) {
        int i = JapaneseImperialCalendar.eras.length - 1;
        while (i > 0) {
            int year = JapaneseImperialCalendar.eras[i].getSinceDate().getYear();
            if (n == year) {
                return true;
            }
            if (n > year) {
                break;
            }
            --i;
        }
        return false;
    }
    
    private static int getEraIndex(LocalGregorianCalendar.Date date) {
        Era era = date.getEra();
        int i = JapaneseImperialCalendar.eras.length - 1;
        while (i > 0) {
            if (JapaneseImperialCalendar.eras[i] == era) {
                return i;
            }
            --i;
        }
        return 0;
    }
    
    private JapaneseImperialCalendar getNormalizedCalendar() {
        if (this.isFullyNormalized()) {
            return this;
        }
        this = (JapaneseImperialCalendar)this.clone();
        this.setLenient(true);
        this.complete();
        return this;
    }
    
    private void pinDayOfMonth(LocalGregorianCalendar.Date date) {
        int year = date.getYear();
        int dayOfMonth = date.getDayOfMonth();
        if (year != this.getMinimum(1)) {
            date.setDayOfMonth(1);
            JapaneseImperialCalendar.jcal.normalize(date);
            int monthLength2 = JapaneseImperialCalendar.jcal.getMonthLength(date);
            if (dayOfMonth > monthLength2) {
                date.setDayOfMonth(monthLength2);
            }
            else {
                date.setDayOfMonth(dayOfMonth);
            }
            JapaneseImperialCalendar.jcal.normalize(date);
        }
        else {
            LocalGregorianCalendar.Date calendarDate = JapaneseImperialCalendar.jcal.getCalendarDate(Long.MIN_VALUE, this.getZone());
            LocalGregorianCalendar.Date calendarDate2 = JapaneseImperialCalendar.jcal.getCalendarDate(this.time, this.getZone());
            long timeOfDay = calendarDate2.getTimeOfDay();
            calendarDate2.addYear(400);
            calendarDate2.setMonth(date.getMonth());
            calendarDate2.setDayOfMonth(1);
            JapaneseImperialCalendar.jcal.normalize(calendarDate2);
            int monthLength = JapaneseImperialCalendar.jcal.getMonthLength(calendarDate2);
            if (dayOfMonth > monthLength) {
                calendarDate2.setDayOfMonth(monthLength);
            }
            else if (dayOfMonth < calendarDate.getDayOfMonth()) {
                calendarDate2.setDayOfMonth(calendarDate.getDayOfMonth());
            }
            else {
                calendarDate2.setDayOfMonth(dayOfMonth);
            }
            if (calendarDate2.getDayOfMonth() == calendarDate.getDayOfMonth() && timeOfDay < calendarDate.getTimeOfDay()) {
                calendarDate2.setDayOfMonth(Math.min(dayOfMonth + 1, monthLength));
            }
            date.setDate(year, calendarDate2.getMonth(), calendarDate2.getDayOfMonth());
        }
    }
    
    private static int getRolledValue(int n, int n2, int n3, int n4) {
        assert n >= n3 && n <= n4;
        int n5 = n4 - n3 + 1;
        n2 %= n5;
        int n6 = n + n2;
        if (n6 > n4) {
            n6 -= n5;
        }
        else if (n6 < n3) {
            n6 += n5;
        }
        assert n6 >= n3 && n6 <= n4;
        return n6;
    }
    
    private int internalGetEra() {
        return this.isSet(0) ? this.internalGet(0) : JapaneseImperialCalendar.currentEra;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.jdate == null) {
            this.jdate = JapaneseImperialCalendar.jcal.newCalendarDate(this.getZone());
            this.cachedFixedDate = Long.MIN_VALUE;
        }
    }
    
    static {
        jcal = (LocalGregorianCalendar)CalendarSystem.forName("japanese");
        gcal = CalendarSystem.getGregorianCalendar();
        BEFORE_MEIJI_ERA = new Era("BeforeMeiji", "BM", Long.MIN_VALUE, false);
        MIN_VALUES = new int[] { 0, -292275055, 0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, -46800000, 0 };
        LEAST_MAX_VALUES = new int[] { 0, 0, 0, 0, 4, 28, 0, 7, 4, 1, 11, 23, 59, 59, 999, 50400000, 1200000 };
        MAX_VALUES = new int[] { 0, 292278994, 11, 53, 6, 31, 366, 7, 6, 1, 11, 23, 59, 59, 999, 50400000, 7200000 };
        Era[] eras2 = JapaneseImperialCalendar.jcal.getEras();
        int n = eras2.length + 1;
        eras = new Era[n];
        sinceFixedDates = new long[n];
        int currentEra2 = 0;
        JapaneseImperialCalendar.sinceFixedDates[currentEra2] = JapaneseImperialCalendar.gcal.getFixedDate(JapaneseImperialCalendar.BEFORE_MEIJI_ERA.getSinceDate());
        JapaneseImperialCalendar.eras[currentEra2++] = JapaneseImperialCalendar.BEFORE_MEIJI_ERA;
        Era era;
        for (int length = eras2.length, i = 0; i < length; JapaneseImperialCalendar.sinceFixedDates[currentEra2] = JapaneseImperialCalendar.gcal.getFixedDate(era.getSinceDate()), JapaneseImperialCalendar.eras[currentEra2++] = era, ++i) {
            era = eras2[i];
            if (era.getSince(TimeZone.NO_TIMEZONE) < System.currentTimeMillis()) {
            }
        }
        currentEra = currentEra2;
        JapaneseImperialCalendar.LEAST_MAX_VALUES[0] = (JapaneseImperialCalendar.MAX_VALUES[0] = JapaneseImperialCalendar.eras.length - 1);
        int min = Integer.MAX_VALUE;
        int n2 = Integer.MAX_VALUE;
        Gregorian.Date calendarDate = JapaneseImperialCalendar.gcal.newCalendarDate(TimeZone.NO_TIMEZONE);
        int year;
        for (int j = 1; j < JapaneseImperialCalendar.eras.length; min = Math.min(year, min), ++j) {
            long n3 = JapaneseImperialCalendar.sinceFixedDates[j];
            CalendarDate sinceDate = JapaneseImperialCalendar.eras[j].getSinceDate();
            calendarDate.setDate(sinceDate.getYear(), 1, 1);
            long fixedDate = JapaneseImperialCalendar.gcal.getFixedDate(calendarDate);
            if (n3 != fixedDate) {
                n2 = Math.min((int)(n3 - fixedDate) + 1, n2);
            }
            calendarDate.setDate(sinceDate.getYear(), 12, 31);
            long fixedDate2 = JapaneseImperialCalendar.gcal.getFixedDate(calendarDate);
            if (n3 != fixedDate2) {
                n2 = Math.min((int)(fixedDate2 - n3) + 1, n2);
            }
            LocalGregorianCalendar.Date calendarDate2 = getCalendarDate(n3 - 1L);
            year = calendarDate2.getYear();
            if (calendarDate2.getMonth() != 1 || calendarDate2.getDayOfMonth() != 1) {
                --year;
            }
        }
        JapaneseImperialCalendar.LEAST_MAX_VALUES[1] = min;
        JapaneseImperialCalendar.LEAST_MAX_VALUES[6] = n2;
    }
}
